from textwrap import dedent
from django.apps import AppConfig, apps
from .column_serializers.column_serializer import (
    ColumnSerializer,
)

from django_model_docs.serializers.model_serializer import (
    ModelSerializer,
)


class AppSerializer:

    """
    Django App serializer

    Generate a markdown document describing a Django app. The document
    consists of an H1 title, description, and a document section for
    each model.
    """

    model_serializer: ModelSerializer
    """Serialize a model as a document section"""

    def __init__(
        self,
        model_serializer: ModelSerializer = None,
        columns: list[ColumnSerializer] = None,
    ):
        """
        Initialize an AppSerializer

        Args:
            model_serializer: Optional ccustom ModelSerializer. If None, uses
                              builtin ModelSerializer
                     columns: A custom set of columns to use when rendering
                              tables. Overrides the columns property of the
                              model_serializer.
        """

        if model_serializer is None:
            self.model_serializer = ModelSerializer()

        if columns is not None:
            self.model_serializer.columns = columns

    @property
    def disclaimer_text(self) -> str:
        """
        Disclaimer added to description.
        """

        return (
            "_This file was auto-generated Django Model Docs Generator."
            + "  Do not edit this file directly._"
        )

    def heading(self, app_name: str) -> str:
        """
        Text to use for the document heading
        """

        return f"`{app_name}`` app`"

    def description(self, app_config: AppConfig) -> str:
        """
        A description of the app
        """

        return str(app_config.module.__doc__).strip()

    def markdown(self, app_name: str) -> str:
        """
        Generate markdown documentation for an app

        Args:
            app_name:str : The name of an installed Django app
        """

        app_config = apps.get_app_config(app_name)

        app_models = app_config.get_models()

        app_documentation = dedent(
            f"# {self.heading(app_name)}" + "\n\n" + f"{self.disclaimer_text}" + "\n\n"
        )

        app_documentation += self.description(app_config) + "\n\n"

        for model in app_models:
            app_documentation += self.model_serializer.markdown(model)

        return app_documentation
